import torch
import torch.nn as nn

class ConvBlock(torch.nn.Module):
    def __init__(self, input_size, output_size, kernel_size, stride, padding):
        super(ConvBlock, self).__init__()
        self.conv0 = nn.Conv3d(input_size, output_size, 3, 1, 1)
        self.bn0 = nn.BatchNorm3d(output_size)
        self.activ0 = nn.LeakyReLU(0.2, True)

        self.conv = nn.Conv3d(output_size, output_size, kernel_size, stride, padding)
        self.bn = nn.BatchNorm3d(output_size)
        self.activ = nn.LeakyReLU(0.2, True)

    def forward(self, x):
        x = self.conv0(x)
        x = self.bn0(x)
        x = self.activ0(x)

        x = self.conv(x)
        x = self.bn(x)
        x = self.activ(x)
        return x

class ConvBlockxBN(torch.nn.Module):
    def __init__(self, input_size, output_size, kernel_size, stride, padding):
        super(ConvBlockxBN, self).__init__()
        self.conv = nn.Conv3d(input_size, output_size, kernel_size, stride, padding)
        # self.bn = nn.BatchNorm3d(output_size)
        self.activ = nn.LeakyReLU(0.2, True)

    def forward(self, x):
        x = self.conv(x)
        # x = self.bn(x)
        # x = self.activ(x)
        return x


class DeconvBlock(torch.nn.Module):
    def __init__(self, input_size, output_size, kernel_size, stride, padding):
        super(DeconvBlock, self).__init__()
        self.conv = nn.ConvTranspose3d(output_size, output_size, kernel_size, stride, padding)
        self.bn = nn.BatchNorm3d(output_size)
        self.activ = nn.ReLU(True)

        self.conv0 = nn.Conv3d(input_size, output_size, 3, 1, 1)
        self.bn0 = nn.BatchNorm3d(output_size)
        self.activ0 = nn.ReLU(True)

    def forward(self, x):
        x = self.conv0(x)
        x = self.bn0(x)
        x = self.activ0(x)


        x = self.conv(x)
        x = self.bn(x)
        x = self.activ(x)
        return x

class DeconvBlockxBN(torch.nn.Module):
    def __init__(self, input_size, output_size, kernel_size, stride, padding):
        super(DeconvBlockxBN, self).__init__()
        self.conv = nn.ConvTranspose3d(input_size, output_size, kernel_size, stride, padding)
        # self.bn = nn.BatchNorm3d(output_size)
        self.activ = nn.ReLU(True)

    def forward(self, x):
        x = self.conv(x)
        # x = self.bn(x)
        # x = self.activ(x)
        return x


class Generator(torch.nn.Module):
    def __init__(self, n_ch):
        super(Generator, self).__init__()

        # Encoder
        self.conv1 = ConvBlock(n_ch, 64, kernel_size=4, stride=2, padding=1)
        self.conv2 = ConvBlock(64, 128, kernel_size=4, stride=2, padding=1)
        self.conv3 = ConvBlock(128, 256, kernel_size=4, stride=2, padding=1)
        self.conv4 = ConvBlock(256, 512, kernel_size=4, stride=2, padding=1)
        # self.conv5 = ConvBlock(256, 512, kernel_size=4, stride=2, padding=1)
        # Decoder
        # self.deconv1 = DeconvBlock(512, 256, kernel_size=4, stride=2, padding=1)
        self.deconv2 = DeconvBlock(512, 256, kernel_size=4, stride=2, padding=1)
        self.deconv3 = DeconvBlock(512, 128, kernel_size=4, stride=2, padding=1)
        self.deconv4 = DeconvBlock(256, 64, kernel_size=4, stride=2, padding=1)
        self.deconv5 = DeconvBlock(128, 16, kernel_size=4, stride=2, padding=1)
        self.deconv6 = DeconvBlockxBN(n_ch+16, 16, kernel_size=3, stride=1, padding=1)

        self.bn = nn.BatchNorm3d(16)
        self.out_conv = nn.ConvTranspose3d(16, 1, 3, 1, 1)


    def forward(self, x):
        # Encoder
        enc1 = self.conv1(x)
        enc2 = self.conv2(enc1)
        enc3 = self.conv3(enc2)
        enc4 = self.conv4(enc3)
        # enc5 = self.conv5(enc4)


        # Decoder with skip-connections
        # dec1 = self.deconv1(enc5)
        # dec1 = torch.cat([dec1, enc4], 1)
        dec2 = self.deconv2(enc4)
        dec2 = torch.cat([dec2, enc3], 1)
        dec3 = self.deconv3(dec2)
        dec3 = torch.cat([dec3, enc2], 1)
        dec4 = self.deconv4(dec3)
        dec4 = torch.cat([dec4, enc1], 1)
        dec5 = self.deconv5(dec4)
        dec5 = torch.cat([dec5, x], 1)
        dec6 = self.deconv6(dec5)
        dec6 = self.bn(dec6)
        out = torch.nn.ReLU()(dec6)
        
        out = self.out_conv(out)

        return out

    def normal_weight_init(self, mean=0.0, std=0.02):
        for m in self.children():
            if isinstance(m, ConvBlock):
                torch.nn.init.normal(m.conv.weight, mean, std)
            if isinstance(m, DeconvBlock):
                torch.nn.init.normal(m.conv.weight, mean, std)




class Discriminator(torch.nn.Module):
    def __init__(self, n_ch):
        super(Discriminator, self).__init__()

        self.conv1 = ConvBlock(n_ch, 16, kernel_size=4, stride=2, padding=1)
        self.conv2 = ConvBlock(16, 32, kernel_size=4, stride=2, padding=1)
        self.conv3 = ConvBlock(32, 64, kernel_size=4, stride=2, padding=1)
        self.conv4 = ConvBlock(64, 64, kernel_size=4, stride=2, padding=1)
        self.conv5 = ConvBlockxBN(64, 1, kernel_size=4, stride=1, padding=1)


    def forward(self, x, label):
        x = torch.cat([x, label], 1)
        x = self.conv1(x)
        x = self.conv2(x)
        x = self.conv3(x)
        x = self.conv4(x)
        x = self.conv5(x)
        out = torch.nn.Sigmoid()(x)
        return out

    def normal_weight_init(self, mean=0.0, std=0.02):
        for m in self.children():
            if isinstance(m, ConvBlock):
                torch.nn.init.normal(m.conv.weight, mean, std)

class DiceLoss(nn.Module):
    def __init__(self):
        super(DiceLoss, self).__init__()

    def	forward(self, input, target):
        N = target.size(0)
        smooth = 1
        th = torch.zeros_like(input).cuda()
        input = torch.where(input > 0, input, th)

        

        input_flat = input.view(N, -1)
        target_flat = target.view(N, -1)

        intersection = input_flat * target_flat

        loss = 2 * (intersection.sum(1) + smooth) / (input_flat.sum(1) + target_flat.sum(1) + smooth)
        loss = 1 - loss.sum() / N

        return loss


if __name__ == "__main__":
    device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
    image_size = 128
    x = torch.Tensor(1, 1, 48, image_size, image_size)
    x = x.cuda()

    x2 = torch.Tensor(1, 2, 32, image_size, image_size)
    x2 = x2.cuda()
    print("x size: {}".format(x.size()))
    
    # x = torch.Tensor(1, 100 )
    # x = x.cuda()

    model= Generator(1)
    model.cuda()
    out  = model(x)
    print("out size: {}".format(out.size()))